package  
{
	import flash.display.Shape;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	
	/**
	 * A player that we can move around
	 * @author Damian Connolly
	 */
	public class Player extends Shape
	{
		
		/********************************************************************************/
		
		private var m_isMoving:Boolean = false; // for when we're moving
		
		/********************************************************************************/
		
		public function Player() 
		{
			this.graphics.lineStyle( 2.0, 0 );
			this.graphics.beginFill( 0x009900 );
			this.graphics.drawCircle( 0.0, 0.0, 10.0 );
			this.graphics.lineTo( 0.0, 0.0 );
			this.graphics.endFill();
			
			// get when we're added to the stage
			this.addEventListener( Event.ADDED_TO_STAGE, this._onAddedToStage );
		}
		
		/**
		 * Updates the player
		 * @param dt The delta time
		 * @param mx The mouse x
		 * @param my The mouse y
		 */
		public function update( dt:Number, mx:Number, my:Number ):void
		{
			// update our rotation
			this.rotation = MathHelper.degreeAngle( mx - this.x, my - this.y );
			
			// check our movement
			if ( !this.m_isMoving )
				return;
			
			// move
			var speed:Number 	= 200;
			var rotRads:Number	= MathHelper.degreesToRadians( this.rotation );
			var dx:Number		= Math.cos( rotRads ) * speed * dt;
			var dy:Number		= Math.sin( rotRads ) * speed * dt;
			this.x += dx;
			this.y += dy;
		}
		
		/********************************************************************************/
		
		// called when we've been added to the stage - add our key listeners
		private function _onAddedToStage( e:Event ):void
		{
			this.addEventListener( Event.ADDED_TO_STAGE, this._onAddedToStage );
			this.stage.addEventListener( KeyboardEvent.KEY_DOWN, this._onKeyDown );
			this.stage.addEventListener( KeyboardEvent.KEY_UP, this._onKeyUp );
		}
		
		// called when a key is pressed down
		private function _onKeyDown( e:KeyboardEvent ):void
		{
			if ( e.keyCode == Keyboard.UP || e.keyCode == Keyboard.W )
				this.m_isMoving = true;
		}
		
		// called when a key is released
		private function _onKeyUp( e:KeyboardEvent ):void
		{
			if ( e.keyCode == Keyboard.UP || e.keyCode == Keyboard.W )
				this.m_isMoving = false;
		}
		
	}

}