package 
{
	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	import flash.display.Shape;
	import flash.events.Event;
	import flash.events.ProgressEvent;
	import flash.utils.getDefinitionByName;
	
	/**
	 * The Preloader loads the game and creates the Main class when we're ready
	 * @author Damian Connolly
	 */
	public class Preloader extends MovieClip 
	{
		
		/*******************************************************************************************/
		
		private var m_bar:Shape = new Shape; // our loading bar
		
		/*******************************************************************************************/
		
		/**
		 * Creates our preloader; adds listeners to see if we're loaded yet
		 */
		public function Preloader() 
		{
			this.addEventListener( Event.ENTER_FRAME, this._checkFrame );
			this.loaderInfo.addEventListener( ProgressEvent.PROGRESS, this._onProgress );
			
			// show loader
			this.m_bar = new Shape;
			this.m_bar.graphics.beginFill( 0x000000 );
			this.m_bar.graphics.drawRect( 0.0, 0.0, 100.0, 2.0 );
			this.m_bar.graphics.endFill();
		}
		
		/*******************************************************************************************/
		
		// called as our loading progresses. update the preloader
		private function _onProgress( e:ProgressEvent ):void 
		{
			if ( this.m_bar.parent == null )
			{
				// position and add our bar
				this.m_bar.x = this.stage.stageWidth * 0.5 - this.m_bar.width * 0.5;
				this.m_bar.y = this.stage.stageHeight * 0.5 - this.m_bar.height * 0.5;
				this.addChild( this.m_bar );
				
				// draw our outline
				this.graphics.lineStyle( 1.0 );
				this.graphics.drawRect( this.m_bar.x - 1.5, this.m_bar.y - 1.5, this.m_bar.width + 3.0, this.m_bar.height + 3.0 );
				
				// set our initial bar scale
				this.m_bar.scaleX = 0.0;
			}
			
			// update loader
			var pc:Number 		= e.bytesLoaded / e.bytesTotal;
			pc					= ( pc < 0.0 ) ? 0.0 : ( pc > 1.0 ) ? 1.0 : pc;
			this.m_bar.scaleX	= pc;
		}
		
		// checks every frame to see if we're fully loaded yet
		private function _checkFrame( e:Event ):void 
		{
			if ( this.currentFrame == this.totalFrames ) 
			{
				this.removeEventListener( Event.ENTER_FRAME, this._checkFrame );
				this._startup();
			}
		}
		
		// searches for the Main class, creates it, and adds it to the stage
		private function _startup():void 
		{
			// hide loader
			this.stop();
			this.loaderInfo.removeEventListener(ProgressEvent.PROGRESS, this._onProgress );
			var mainClass:Class = getDefinitionByName("Main") as Class;
			this.stage.addChild( new mainClass() as DisplayObject );
			this.stage.removeChild( this );
		}
		
	}
	
}